# Generated by Django 3.2.6 on 2021-09-06 15:05

import django.core.files.storage
from django.db import migrations, models
import paper_uploads.models.fields.image
import paper_uploads.models.utils


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads', '0002_auto_20210407_0627'),
    ]

    operations = [
        migrations.AlterField(
            model_name='fileitem',
            name='file',
            field=models.FileField(max_length=255, storage=django.core.files.storage.FileSystemStorage(), upload_to=paper_uploads.models.utils.generate_filename, verbose_name='file'),
        ),
        migrations.AlterField(
            model_name='imageitem',
            name='file',
            field=paper_uploads.models.fields.image.VariationalFileField(max_length=255, storage=django.core.files.storage.FileSystemStorage(), upload_to=paper_uploads.models.utils.generate_filename, verbose_name='file'),
        ),
        migrations.AlterField(
            model_name='svgitem',
            name='file',
            field=models.FileField(max_length=255, storage=django.core.files.storage.FileSystemStorage(), upload_to=paper_uploads.models.utils.generate_filename, verbose_name='file'),
        ),
        migrations.AlterField(
            model_name='uploadedfile',
            name='file',
            field=models.FileField(max_length=255, storage=django.core.files.storage.FileSystemStorage(), upload_to=paper_uploads.models.utils.generate_filename, verbose_name='file'),
        ),
        migrations.AlterField(
            model_name='uploadedimage',
            name='file',
            field=paper_uploads.models.fields.image.VariationalFileField(max_length=255, storage=django.core.files.storage.FileSystemStorage(), upload_to=paper_uploads.models.utils.generate_filename, verbose_name='file'),
        ),
    ]
