# Generated by Django 3.2.9 on 2021-11-06 10:04

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('paper_uploads', '0003_auto_20210906_1505'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='fileitem',
            options={'default_permissions': ()},
        ),
        migrations.AlterModelOptions(
            name='imageitem',
            options={'default_permissions': ()},
        ),
        migrations.AlterModelOptions(
            name='svgitem',
            options={'default_permissions': ()},
        ),
    ]
