# Generated by Django 3.2.9 on 2021-11-16 05:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('paper_uploads', '0004_auto_20211106_1004'),
    ]

    operations = [
        migrations.AlterIndexTogether(
            name='collectionitembase',
            index_together={('collection_id', 'collection_content_type')},
        ),
    ]
